import os
import unittest
from pathlib import Path
from unittest.mock import patch

from obspy.io.reftek.packet import PACKET_FINAL_DTYPE

from sohstationviewer.model.mseed_data.record_reader_helper import Unpacker
from sohstationviewer.model.reftek.reftek_reader.header import NotRT130FileError
from sohstationviewer.model.reftek.reftek_reader.packet_readers import (
    read_eh_et_packet, read_dt_packet, read_soh_packet,
)
from sohstationviewer.model.reftek.reftek_reader.packets import (
    SOHPacket,
    EHETPacket, DTPacket,
)
from sohstationviewer.model.reftek.reftek_reader.reftek_helper import (
    read_rt130_file, convert_packet_to_obspy_format,
)

unpacker = Unpacker('>')


class TestReadRT130File(unittest.TestCase):
    def setUp(self) -> None:
        self.TEST_DATA_DIR = Path(os.getcwd()).joinpath('tests/test_data')
        self.rt130_dir = self.TEST_DATA_DIR.joinpath(
            'RT130-sample/2017149.92EB/2017150/92EB'
        )

        eh_et_patcher = patch(
            'sohstationviewer.model.reftek.reftek_reader.reftek_helper.'
            'read_eh_et_packet',
            wraps=read_eh_et_packet
        )
        self.mock_read_eh_et = eh_et_patcher.start()
        self.addCleanup(eh_et_patcher.stop)

        dt_patcher = patch(
            'sohstationviewer.model.reftek.reftek_reader.reftek_helper.'
            'read_dt_packet',
            wraps=read_dt_packet
        )
        self.mock_read_dt = dt_patcher.start()
        self.addCleanup(dt_patcher.stop)

        soh_patcher = patch(
            'sohstationviewer.model.reftek.reftek_reader.reftek_helper.'
            'read_soh_packet',
            wraps=read_soh_packet
        )
        self.mock_read_soh = soh_patcher.start()
        self.addCleanup(soh_patcher.stop)

    def test_rt130_soh_file(self):
        file = self.rt130_dir.joinpath('0/000000000_00000000')
        packets = read_rt130_file(file, unpacker)
        self.assertTrue(
            all(isinstance(packet, SOHPacket) for packet in packets)
        )
        self.assertTrue(self.mock_read_soh.called)
        self.assertFalse(self.mock_read_dt.called)
        self.assertFalse(self.mock_read_eh_et.called)

    def test_rt130_raw_data_file(self):
        file = self.rt130_dir.joinpath('1/000000015_0036EE80')
        packets = read_rt130_file(file, unpacker)
        self.assertTrue(all(
            isinstance(packet, EHETPacket) or isinstance(packet, DTPacket)
            for packet in packets)
        )
        self.assertFalse(self.mock_read_soh.called)
        self.assertTrue(self.mock_read_dt.called)
        self.assertTrue(self.mock_read_eh_et.called)

    def test_non_rt130_file(self):
        with self.subTest('test_file_exist'):
            file = self.TEST_DATA_DIR.joinpath(
                'Q330-sample/day_vols_AX08/AX08.XA..HHE.2021.186'
            )
            with self.assertRaises(NotRT130FileError):
                read_rt130_file(file, unpacker)

        with self.subTest('test_file_does_not_exist'):
            file = ''
            with self.assertRaises(FileNotFoundError):
                read_rt130_file(file, unpacker)


class TestConvertPacketToObspyFormat(unittest.TestCase):
    def setUp(self) -> None:
        TEST_DATA_DIR = Path(os.getcwd()).joinpath('tests/test_data')
        rt130_dir = TEST_DATA_DIR.joinpath(
            'RT130-sample/2017149.92EB/2017150/92EB'
        )
        file = rt130_dir.joinpath('1/000000015_0036EE80')
        self.packet = read_rt130_file(file, unpacker)[0]

    def test_all_needed_fields_are_available(self):
        converted_packet = convert_packet_to_obspy_format(
            self.packet, unpacker
        )

        self.assertEqual(len(converted_packet), len(PACKET_FINAL_DTYPE))
